#!/usr/bin/perl -w

###########################################
# Program : auto_primer1.0.pl
# Auto-Primer ver 1.00 - Dec. 6, 2012
# Copyright (c) 2012, NRIFS-FRA
# FRA-bioinfo@ml.affrc.go.jp
# http://nrifs.fra.affrc.go.jp/index.html
###########################################

my $date = ` date `; 

my $bin_dir = "/home/auto_primer/bin" ; 

my $trf_exe = "trf404.linux64.exe" ;
my $pr3_exe = "/usr/local/primer3-2.2.2-beta/bin/primer3_core" ;
my $extract_flanking = $bin_dir."/extract_flanking.pl" ;
my $extract_primers = $bin_dir."/extract_primers.pl" ;
my $split_fasta = $bin_dir."/split_fasta.pl" ;

my @execs = ( $trf_exe, $extract_flanking, $pr3_exe, $extract_primers, $split_fasta ) ;
foreach ( @execs ) { 
	if ( ! ` which $_ ` ) { die  "$_ : Cannot execute. PATH is correct?\n" ; }
}
if ( @ARGV == 0 ) { print  "Just checking PATH ... OK.\n" ; } 

my $min_unit = 2 ; 
my $max_unit = 5 ; 
my $min_product = 100 ;
my $max_product = 400 ;
my $min_flanking = 30 ; 

my $seq_limit = 5000 ;
 
if ( @ARGV < 2 || grep { /\-help/ } @ARGV ) { 
	warn  "Usage : $0  IN_DIR_or_FASTA  OUT_STEM  (OPTIONS)\n\n" ; 

	warn  "IN_DIR_or_FASTA : An input directory including FASTA files or a FASTA file\n" ; 
	warn  "OUT_STEM  : An output name. The list files of PCR primers will be OUT_STEM_2bp.xls, OUT_STEM_3bp.xls, etc.\n" ; 
	warn  "            A log file will be OUT_STEM_log.txt\n" ; 
	warn  "            A summary file for each repeat will be OUT_STEM_rep.xls\n" ; 
	warn  "            A sequence file for all MSs will be OUT_STEM_seq.fna\n" ; 
	warn  "            A detected MS list file will be OUT_STEM_list.xls\n" ; 
	warn  "\n" ; 
	warn  "OPTIONS :\n" ; 
	warn  "-tmp DIRECTORY : A directory to save intermediate files in error check. (Unecessary)\n" ;
	warn  "-(min|max): Minimum/Maximum repeat unit length (bp). Default is 2 and 5, respectively.\n" ; 
	warn  "-(min_product|max_product): Minimum/Maximum amplified length by PCR (bp). Default is 100 and 400, respectively.\n" ; 
	warn  "-noqual  : If you don't have any Roche's quality file, use this option.\n" ; 
	warn  "-only_repeat or -noprimer : Repeats found by trf will be shown, and primers are not designed.\n" ; 
	warn  "-notable : MS list \(OUT_STEM_list.xls\) is not output.\n" ; 
	warn  "-nofasta : MS sequence \(OUT_STEM_seq.fna\) is not output.\n" ;
	warn  "-extra_table : MS sequence with no primers detected is output into OUT_STEM_seq_noprimer.fna.\n" ;  
	warn  "-redundant   : Identical primers in different contigs are permitted.\n" ; 
	exit ( 0 ) ; 
} 

my $in_dir   = shift(@ARGV) ;
my $out_stem = shift(@ARGV) ;
if ( $out_stem =~ /^(.+)\.\S+/ ) { $out_stem = $1 ; }

my $log = $out_stem.".log" ;
my $rep_stat = $out_stem."_pat.xls" ;
my $fasta = $out_stem.".fna" ;
my $extra_fasta = $out_stem."_extra.fna" ;
my $table = $out_stem."_all.xls" ; 
my $error = $out_stem.".err" ; 		## If this file is created, some errors happened.
if ( $fasta eq $in_dir ) { die  "Query FASTA is identical to the output FASTA.\n" ; } 

my $qual = 1 ;
my $out_fasta = 1 ;		## Output FASTA
my $out_extra_fasta = 0 ;		## Not output extra FASTA
my $out_table = 1 ;		## Output table
my $out_primer = 1 ;	## Primers will be also detected
my $intermediate ;

my $red_check = 1 ;	
my $red_min = 10 ; 
my $red_primer = {} ;	## $red_primer->{PRIEMR_SEQ} in &redudancy_check
my %comp_base = (  'A' => 'T' , 'C' => 'G' , 'G' => 'C' , 'T' => 'A' ,
                   'R' => 'Y' , 'Y' => 'R' , 'M' => 'K' ,
                   'K' => 'M' , 'S' => 'S' , 'W' => 'W' ,
                   'H' => 'D' , 'B' => 'V' , 'V' => 'B' , 'D' => 'H' ,
                   'N' => 'N' );


while ( $opt = shift(@ARGV) ) { 
	if ( $opt eq "-noqual" ) { 
		$qual = 0 ; 
	} elsif ( $opt eq "-min" && @ARGV > 0 ) { 
		$min_unit = shift(@ARGV) ; 
	} elsif ( $opt eq "-max" && @ARGV > 0 ) {
		$max_unit = shift(@ARGV) ;
	} elsif ( $opt eq "-min_product" && @ARGV > 0 ) { 
		$min_product = shift(@ARGV) ;
	} elsif ( $opt eq "-max_product" && @ARGV > 0 ) {
		$max_product = shift(@ARGV) ;
	} elsif ( $opt eq "-tmp" && @ARGV > 0 ) {
		$intermediate = shift(@ARGV) ;
	} elsif ( $opt eq "-only_repeat" || $opt eq "-noprimer" ) {
		$out_primer = 0 ;
		$qual = 0 ; 
	} elsif ( $opt eq "-notable" ) {
		$out_table = 0 ;
	} elsif ( $opt eq "-nofasta" ) {
		$out_fasta = 0 ;
	} elsif ( $opt eq "-extra_fasta" ) {
		$out_extra_fasta = 1 ;
	} elsif ( $opt eq "-redundant" ) {
		$red_check = 0 ;
	} else { 
		die  "$a : Incorrect option\n" ; 
	}
} 
if ( $min_unit > $max_unit ) { die  "Error : MIN_UNIT > MAX_UNIT\n" ; } 
if ( $min_product > $max_product ) { die  "Error : MIN_PRODUCT > MAX_PRODUCT\n" ; } 
my $ex1_option = "-round -min_flanking 0 -min_product $min_product -max_product $max_product" ; 
my $ex2_option = "-min_product $min_product -max_product $max_product" ; 
if ( $out_fasta == 0 ) { $out_extra_fasta = 0 ; }

my $trf_option = "2 7 7 80 10 50 $max_unit -f -d -m" ;
my $pr3_option = "-format_output" ;


my @trf_out ; 
my $tmp_dir = "tmp".$$ ; 
my $cur = ` pwd ` ;
chomp ( $cur ) ;

if ( defined $intermediate ) { 
	$intermediate = $cur."/".$intermediate ;
	if ( ! -d $intermediate ) { 
		mkdir ( $intermediate, 0755 ) or die "$intermediate cannot be created.\n" ; 
	}
	$im_trf = $intermediate."/trf" ;
	$im_pr3 = $intermediate."/primer3" ;
	if ( -d $im_trf ) { system ( "rm -rf $im_trf" ) ; }
	if ( -d $im_pr3 ) { system ( "rm -rf $im_pr3" ) ; }
	mkdir ( $im_trf, 0755 ) ;
	mkdir ( $im_pr3, 0755 ) ;
}


my @seq_files ; 
if ( -d $in_dir ) { 
	$in_dir =~ s/\/$// ;
	@seq_files = ` ls $in_dir ` ;
	@seq_files = grep { /\.(fa|fna|fasta)(\.Z|\.gz)?$/ } @seq_files ; 
	@seq_files = grep { ! /454LargeContigs/ } @seq_files ;
	if ( @seq_files == 0 ) { die  "$in_dir : No FASTA file.\n" ; }
	chomp ( @seq_files ) ;
	foreach ( @seq_files ) { $_ = $in_dir."/".$_ ; } 
} elsif ( -f $in_dir ) { 
	$seq_files[0] = $in_dir ;
} else { 
	die  "$in_dir : Incorrect input.\n" ; 
}

if ( -d $tmp_dir ) { system ( "rm -rf $tmp_dir" ) ; } 


open ( LOG, ">$log" ) or die "$log : Cannot open.\n" ; 
print  LOG  "# Executed program : $0\n" ; 
print  LOG  "# Start Date : $date" ; 
print  LOG  "\n" ; 
print  LOG  "# Executed sub-programs :\n" ; 
foreach ( @execs ) { print  LOG  "# $_\n" ; }
print  LOG  "\n" ; 
print  LOG  "# trf option         : $trf_option\n" ;  
print  LOG  "# Extract-trf option : $ex1_option\n" ; 
print  LOG  "# primer3 option     : $pr3_option\n" ;
print  LOG  "# Extract-primer3 option : $ex2_option\n" ; 
if ( $red_check == 1 ) { 
	print  LOG  "# Redundant primers are excluded.\n" ; 
} else { 
	print  LOG  "# Redundant primers are included.\n" ; 
}
 
print  LOG  "\n" ; 
print  LOG  "# Examined FASTA files :\n" ; 
my @qual_files ;
my $num = 1 ; 
foreach ( @seq_files ) { 
	print  LOG  "# $num\. ".$_."\n" ; 
	if ( $qual > 0 ) { 
		/^(\S+)\.(fa|fna|fasta)/ ;
		$s = $1.".qual" ; 
		#if ( ! -f $s ) { die  "$s : Cannot open. If you don\'t use .qual files, add an option \"-noqual\"\n" ; } 
		if ( -s $s ) { 
			push ( @qual_files, $s ) ; 
		} else { 
			push ( @qual_files, "-" ) ; 
		}
	}
	$num++ ;
}
if ( $qual > 0 ) {  
	print  LOG  "\n" ; 
	print  LOG  "# Quality files referred :\n" ; 
	$num = 1 ; 
	foreach ( @qual_files ) { 
		print  LOG  "# $num\. ".$_."\n" ; 
		$num++ ;
	}
}
print  LOG  "\n" ; 
close ( LOG ) ; 


my $out_hash = {} ;
my $pr3_found = {} ;
my $trf_found = {} ;
for ( $i = $min_unit ; $i < $max_unit + 1 ; $i++ ) { 
	if ( $out_primer == 1 ) {
		$out = $out_stem."_".$i."bp.xls" ; 
		$out_hash->{$i} = $out ; 
		open ( OUT, ">$out" ) ; 
		print  OUT  "# FASTA file\tContig ID\tRepeat start position (5\')\tRepeat sequence\tSimple notation\tRepeat type\tPrimer ID\tPrimer sequences (5\'-3\')\tPrimer Tm\tExpected allele size (bp)\tPrimer end position (3\')" ;
		print  OUT  "\tProb. of No error\tProb. of at most one error\tMean of Q-scores of amplified sequence" ;
		print  OUT  "\n" ; 
		close ( OUT ) ;
	}
	$pr3_found->{$i} = 0 ; 
	$trf_found->{$i} = 0 ;
}


if ( -f $error ) { unlink ( $error ) ; } 

my @output ;
my $pr3_total = 0 ; 
my $trf_total = 0 ; 
my $each_rep_trf_count = {} ;
my $each_rep_pr3_count = {} ;
for ( $i = 0 ; $i < @seq_files ; $i++ ) { 

	$s = $seq_files[$i] ;

	### Too much sequences (> $seq_limit)
	if ( $s =~ /\.(Z|gz)$/ ) { 
		$total_seq = ` zgrep -c  \'^\>\' $s ` ;
	} else {
		$total_seq = ` grep -c  \'^\>\' $s ` ;
	}
	chomp ( $total_seq ) ; 
	if ( $total_seq > $seq_limit ) {
		$all_sections = int($total_seq/$seq_limit) + 1 ; 
	} else {
		$all_sections = 1 ; 
	}


	@output = () ; 
	@fasta_list = () ;
	@extra_fasta_list = () ;
	@rep_list = () ; 

	$name_hash = {} ;
	$trf_available_total = 0 ;
	$trf_total_html = 0 ; 

	$each_file_trf_count = {} ;
	$each_file_pr3_count = {} ;
	$each_rep_trf_count_part = {} ;
	$each_rep_pr3_count_part = {} ;
	### Initialization for each fasta 
	for ( $u = $min_unit ; $u < $max_unit + 1 ; $u++ ) {
		$each_file_trf_count->{$u} = 0 ;
		$each_file_pr3_count->{$u} = 0 ; 
	}

	for ( $j = 0 ; $j < $all_sections ; $j++ ) { 

		if ( ! -d $tmp_dir ) { mkdir ( $tmp_dir, 0755 ) ; } 
		if ( $all_sections == 1 ) { 
			system ( "cp $s $tmp_dir\/query.fna" ) ; 
		} else {
			system ( "$execs[4]  $s $all_sections $j > $tmp_dir\/query.fna" ) ; 
		}

		chdir ( $tmp_dir ) ; 

		$command = "$execs[0]  query.fna  $trf_option" ; 
		print  $command."\n" ; 
		system ( $command ) ; 

		@trf_out = ` ls ` ; 
		chomp ( @trf_out ) ;
		@trf_out = grep { /\.txt\.html$/ } @trf_out ; 
		$trf_total_html += $#trf_out + 1 ;
		system ( "mkdir primer3_input" ) ;

	
	
		foreach $t ( @trf_out ) { 
			$command = "perl $execs[1] $t primer3_input $ex1_option" ; 
			#print  $command."\n" ;
			@data = ` $command ` ; 
			chomp ( @data ) ; 
			if ( @data < 1 ) {
				open ( ERR, ">$error" ) ; 
				print  ERR  "$execs[1] is failed. Format is different.?\n" ;
				close ( ERR ) ;  
				die  "$execs[1] is failed. Format is different.?\n" ;
			}
			if ( defined $intermediate ) { 
				$mv = $im_trf."/".$data[0].".txt.html" ; 
				system ( "cp $t $mv" ) ; 
			}
			if ( @data < 2 ) { 
				warn  "$t : No repeat found.\n" ;
				next ; 
			}
		
			# $data[0] : e.g. contig00129__length=569___numreads=6
			
			$name_hash->{$data[0]} = 1 ; 
			
			$name = $data[0] ;
			if ( $name =~ /^([^\ \_]+\_(rep_)?(s|c)\d+)/ ) { 		### Mira assemble
				$name = $1 ;
			} elsif ( $name =~ /contig(\d+)/i ) { 
				$name = $1 ;
			} elsif ( $name =~ /^[^\ \_]+\_\d+/ ) { 
				$name = $& ;
			} else { 
				$name =~ /^[^\ \_]+/ ;
				$name = $& ;
				#if ( $name =~ /(\d+)$/ ) { $name = $1 ; } 
				### If contigNNNNNN, later delete "contig" in Excel
			}  


			foreach $d ( @data[1..$#data] ) { 
				#print  "#".$d."\n" ; 

				@vals = split ( /\t/, $d ) ; 

				$unit = $vals[6] ;
				if ( $unit < $min_unit || $unit > $max_unit ) { next ; } 

				$trf_available_total++ ;
				$each_file_trf_count->{$unit}++ ;
				$trf_found->{$unit}++ ;		# Repeat found by TRF (might not be used for primer prediction)


				$rep_start = $vals[2] ; 
				$rep_len = $vals[3] ; 
				$rep_seq = $vals[4] ;
				$rep_seq_sum = $vals[5] ;		# e.g., (ATGCG)5.2
				$judge = $vals[7] ;
				$rep_seq_sum =~ /\((\S+)\)/ ;
				$period_seq = $1 ; 
				$full = $vals[9] ;		### Full sequence including flanking regions

				if ( ! defined $each_rep_trf_count->{$period_seq} ) { 
					$each_rep_trf_count->{$period_seq} = 0 ; 
					$each_rep_pr3_count->{$period_seq} = 0 ;
				} 
				if ( ! defined $each_rep_trf_count_part->{$period_seq} ) { 
					$each_rep_trf_count_part->{$period_seq} = 0 ; 
					$each_rep_pr3_count_part->{$period_seq} = 0 ;
				}
				$each_rep_trf_count->{$period_seq}++ ;
				$each_rep_trf_count_part->{$period_seq}++ ;


				$rep_id = $name."_".$rep_start ;
				$result = join ( "\t", ($s, $data[0], $rep_id, @vals[4..7] ) ) ;

				if ( $out_primer == 1 ) { 

					### Prediction of primer sequences

					if ( $vals[8] < $min_flanking ) { 
						push ( @extra_fasta_list, ">".$rep_id."\t".$rep_seq_sum."\t".$judge."\tShort flanking\n".$full."\n" ) ; 
						push ( @rep_list, $result."\t$rep_len\tShort flanking" ) ; 
						next ; 
					} 

					$command = "$execs[2] $pr3_option $vals[0] \| $execs[3] $ex2_option" ; 
					#print  $command."\n" ;
					@primers = ` $command ` ;

					if ( defined $intermediate ) { 
						$mv = $im_pr3."/".$data[0]."__".$rep_id.".seq" ; 
						system ( "cp $vals[0] $mv" ) ; 
					}

					if ( @primers == 0 ) { 
						warn  "$vals[0] : No primer found.\n" ; 
						push ( @extra_fasta_list, ">".$rep_id."\t".$rep_seq_sum."\t".$judge."\tNo primer found\n".$full."\n" ) ;
						push ( @rep_list, $result."\t$rep_len\tNo primer found" ) ; 
						next ; 
					}
					chomp ( @primers ) ;
					$primers[0] =~ /^(\d+)\s+(\d+)\s+(\d+)\s+/ or next ;			
					$amp_size = $1 ; 
					$left = $2 - 1 ; 
					$right = $3 - 1 ;
					$tm_and_seq = $' ;

					if ( $tm_and_seq =~ /n/i ) { next ; }  	### 'N' is included in primer sequences

					$tm_and_seq =~ /\S+\s+\S+\s+(\S+)\s+(\S+)/ ;
					$left_pr = $1 ;
					$right_pr = $2 ; 

					### Primer redudancy check
					if ( $red_check == 1 ) { 
						if ( &redundancy_check ( $left_pr ) == 1 ) { next ; } 
						if ( &redundancy_check ( $right_pr ) == 1 ) { next ; } 
					}
			
					$left_start = $left - length($left_pr) + 1 ;
					$right_end = $right + length($right_pr) - 1 ;
					for ( $k = $left_start ; $k < $left + 1 ; $k++ ) { 
						substr ( $full, $k, 1 ) = uc( substr ( $full, $k, 1 ) ) ; 
					}
					for ( $k = $right ; $k < $right_end + 1 ; $k++ ) { 
						substr ( $full, $k, 1 ) = uc( substr ( $full, $k, 1 ) ) ;
					}
					$rep_start -= $vals[1] ;					
					$tmp_seq = $rep_seq ;
					$tmp_seq =~ s/\-//g ;

					if ( $full =~ /^(\S{$rep_start})$tmp_seq(\S+)/ ) { 
						$full = $1."\n".$tmp_seq."\n".$2 ;
					} else {
						open ( ERR, ">$error" ) ;
						print  ERR  "$full\n\n$rep_start\t$tmp_seq\nError\n" ;
						close ( ERR ) ;
						die  "$full\n\n$rep_start\t$tmp_seq\nError\n" ; 
					}

					$left += $vals[1] ; 
					$right += $vals[1] ;

					push ( @output, $result."\t".$amp_size."\t".$left."\t".$right."\t".$tm_and_seq ) ; 
					push ( @rep_list, $result."\t$rep_len\tDetected" ) ; 
				} else {
					push ( @rep_list, $result."\t$rep_len" ) ; 
				}

				push ( @fasta_list, ">".$rep_id."\t".$rep_seq_sum."\t".$judge."\n".$full."\n" ) ;
			}
		}

		chdir ( $cur ) ; 
		system ( "rm -rf $tmp_dir" ) ;

	}		### End of "for ( $j = 0 ; $j < $all_sections ; $j++ ) { "


	if ( $out_fasta == 1 && @fasta_list > 0 ) { 
		if ( $i == 0 ) { 
			open ( OUT, ">$fasta" ) ;
		} else { 
			open ( OUT, ">>$fasta" ) ;
		}
		foreach ( @fasta_list ) { print  OUT  $_."\n" ; } 
		close ( OUT ) ; 
	}

	if ( $out_extra_fasta == 1 && $out_primer == 1 && @extra_fasta_list > 0 ) { 
		if ( $i == 0 ) { 
			open ( OUT, ">$extra_fasta" ) ;
		} else {
			open ( OUT, ">>$extra_fasta" ) ;
		}
		foreach ( @extra_fasta_list ) { print  OUT  $_."\n" ; } 
		close ( OUT ) ; 
	}

	if ( $out_table == 1 && @rep_list > 0 ) { 
		if ( $i == 0 ) { 
			open ( OUT, ">$table" ) ; 
			print  OUT  "# Sequence file\tSequecne ID\tRepeat ID\tRepeat sequence\tSimple notation\tPeriod\tRepeat type\tRepeat length\tDescription\n" ; 
		} else {
			open ( OUT, ">>$table" ) ; 
		}
		foreach ( @rep_list ) { print  OUT  $_."\n" ; } 
		close ( OUT ) ;
	}


	open ( LOG, ">>$log" ) ; 
	$num = $i + 1 ;
	print  LOG  "# ------------------------------------------------------\n" ; 
	print  LOG  "# $num\. ".$s."\n" ; 
	print  LOG  "# trf .txt.html files  : $trf_total_html\n" ; 
	for ( $u = $min_unit ; $u < $max_unit + 1 ; $u++ ) { 
		print  LOG  "# $u\-bp repeats found : $each_file_trf_count->{$u}\n" ;
	}
	print  LOG  "# Repeats found by trf : $trf_available_total\n" ; 
	print  LOG  "\n" ; 
	close ( LOG ) ; 
	
	$trf_total += $trf_available_total ;


	if ( $out_primer == 0 ) { next ; } 
 

	### Refer Quality file
	if ( $qual > 0 && $qual_files[$i] ne "-" ) { 
		open ( IN, "$qual_files[$i]" ) ;
		$flag = 0 ;
		while ( <IN> ) { 
			chomp ; 
			if ( /^\>(\S.*\S)/ ) { 
				$name = $1 ; 
				$name =~ s/\s+/\_/g ; 
				if ( defined $name_hash->{$name} ) { 
					$flag = 1 ; 
					$name_hash->{$name} = "" ; 
				} else { 
					$flag = 0 ; 
				} 				
			} elsif ( /\d+/ ) { 
				if ( $flag == 0 ) { next ; } 
				$name_hash->{$name} .= $_." " ;
			}
		}
		close ( IN ) ;
	}

	foreach ( @output ) { 

		@VALS = split ( /\t/, $_ ) ;

		if ( defined $name_hash->{$VALS[1]} && $name_hash->{$VALS[1]} ne "1" ) { 
			$name_hash->{$VALS[1]} =~ s/\s+$// ;
#print  $VALS[1]."\n".$name_hash->{$VALS[1]}."\n" ; 
			@scores = split ( /\s+/, $name_hash->{$VALS[1]} ) ; 
			#$left = $VALS[8] ;
			$left = $VALS[8] - 1 ;
			$right = $VALS[9] - 1 ;
			$left_start = $left - length($VALS[12]) + 1 ;
			#$right = $VALS[9] ;
			$right_end = $right + length($VALS[13]) - 1 ;

			@L_scores = () ;
			@R_scores = () ;
			@amp_scores = () ;
			$mean_L = 0 ; 
			$mean_R = 0 ; 
			$mean_A = 0 ; 
			$no_err_L = 0 ; 
			$no_err_R = 0 ; 
			$no_err_A = 0 ; 
			#for ( $k = $left_start - 1 ; $k < $left ; $k++ ) { 
			for ( $k = $right ; $k < $right_end + 1 ; $k++ ) { 
				$mean_L += $scores[$k] ; 
				if ( $scores[$k] < 1 ) { $scores[$k] = 1 ; } 
				$prob = log(1-10**(-0.1 * $scores[$k])) ;
				$no_err_L += $prob ;
				push ( @L_scores, $prob ) ;
			}
			$k = @L_scores ;
			$mean_L /= $k ; 
			#for ( $k = $right - 1 ; $k < $right_end ; $k++ ) { 
			for ( $k = $right ; $k < $right_end + 1 ; $k++ ) { 
				$mean_R += $scores[$k] ; 
				if ( $scores[$k] < 1 ) { $scores[$k] = 1 ; } 
				$prob = log(1-10**(-0.1 * $scores[$k])) ;
				$no_err_R += $prob ;
				push ( @R_scores, $prob ) ;
			}
			$k = @R_scores ;
			$mean_R /= $k ; 
			#for ( $k = $left ; $k < $right - 1 ; $k++ ) { 
			for ( $k = $left + 1 ; $k < $right ; $k++ ) { 
				$mean_A += $scores[$k] ; 
				if ( $scores[$k] < 1 ) { $scores[$k] = 1 ; } 
				$prob = log(1-10**(-0.1 * $scores[$k])) ;
				$no_err_A += $prob ;
				push ( @amp_scores, $prob ) ;
			}
			$k = @amp_scores ;
			$mean_A /= $k ; 

			$at_least_one_err_L = exp($no_err_L) ; 
			$at_least_one_err_R = exp($no_err_R) ;
			foreach $p ( @L_scores ) { 
				$at_least_one_err_L += exp( $no_err_L - $p + log(1-exp($p)) ) ;
			}
			foreach $p ( @R_scores ) { 
				$at_least_one_err_R += exp( $no_err_R - $p + log(1-exp($p)) ) ;
			}
			

			$no_err_L = sprintf ( "%5.2f", 100*exp($no_err_L))  ; 
			$no_err_R = sprintf ( "%5.2f", 100*exp($no_err_R))  ; 
			$no_err_A = sprintf ( "%5.2f", 100*exp($no_err_A))  ; 

			$at_least_one_err_L = sprintf ( "%5.2f", 100*$at_least_one_err_L )  ; 
			$at_least_one_err_R = sprintf ( "%5.2f", 100*$at_least_one_err_R )  ; 

			$mean_A = sprintf ( "%5.2f", $mean_A ) ;
			
		}

		
		### Output primer information

		$out = $out_hash->{$VALS[5]} ;
		open ( OUT, ">>$out" ) ; 
		$VALS[2] =~ /^(\S+)\_(\d+)/ ;
		print  OUT  join  ( "\t", (($VALS[0], $1, $2) , @VALS[3,4,6]) ) ;
		
		print  OUT  "\t".$VALS[2]."_F" ;
		print  OUT  "\t$VALS[12]\t$VALS[10]\t$VALS[7]\t$VALS[8]" ;

		if ( defined $name_hash->{$VALS[1]} && $name_hash->{$VALS[1]} ne "1" ) { 
			print  OUT  "\t".$no_err_L ;
			print  OUT  "\t".$at_least_one_err_L ;
			print  OUT  "\t".$mean_A ;
		}
		print  OUT  "\n" ; 

		print  OUT  "\t\t\t\t\t\t".$VALS[2]."_R" ;
		print  OUT  "\t$VALS[13]\t$VALS[11]\t\t$VALS[9]" ;
		if ( defined $name_hash->{$VALS[1]} && $name_hash->{$VALS[1]} ne "1" ) { 
			print  OUT  "\t".$no_err_R ;
			print  OUT  "\t".$at_least_one_err_R ;
		}
		print  OUT  "\n" ; 
		$each_file_pr3_count->{$VALS[5]}++ ;
		$pr3_found->{$VALS[5]}++ ;

		$VALS[4] =~ /\((\S+)\)/ ;
		$period_seq = $1 ; 
		$each_rep_pr3_count->{$period_seq}++ ;
		$each_rep_pr3_count_part->{$period_seq}++ ;

		close ( OUT ) ;
	}
	
	$pr3_available_total = @output ;

	open ( LOG, ">>$log" ) ; 
	for ( $u = $min_unit ; $u < $max_unit + 1 ; $u++ ) { 
		print  LOG  "# $u\-bp repeat primers designed : $each_file_pr3_count->{$u}\n" ;
	}
	print  LOG  "# Primers designed by primer3 : $pr3_available_total\n" ; 
	print  LOG  "\n" ; 
	close ( LOG ) ; 

	$pr3_total += $pr3_available_total ; 


#last ; 
}


### Append some headers
if ( $out_primer == 1 ) {
	for ( $i = $min_unit ; $i < $max_unit + 1 ; $i++ ) { 
		$out = $out_hash->{$i} ;
		@data = ` grep \'^[^\#\ ]\' $out ` ;
		if ( @data == 0 ) { 
			print  STDERR  "$i\-bp repeat : No found.\n" ; 
			system ( "rm $out" ) ; 
			next ; 
		} 
		system ( "mv  $out  $out\.tmp" ) ; 
		open ( OUT, ">$out" ) ; 
		print  OUT  "# Contig_ID: Last 5 digits of contig name\n" ; 
		print  OUT  "# Primer_ID: Contig_ID + repeat start position + Forward|Reverse\n" ; 
		print  OUT  "# e.g. contig01151_length=603_numreads=5, Start from 137bp and Forward -> Contig_ID=01151, Primer_ID=01151_137_F\n" ; 
		#print  OUT  "# Total primer pair candidates : $pr3_total\n" ; 
		print  OUT  "# Total $i\-bp repeats found by TRF : $trf_found->{$i}\n" ; 
		print  OUT  "# Primer pair candidates of $i\-bp repeat : $pr3_found->{$i}\n" ; 
		print  OUT  "\n" ; 
		close ( OUT ) ;
		system ( "cat  $out\.tmp >>  $out" ) ; 
		system ( "rm  $out\.tmp" ) ; 
	}
}


open ( LOGB, ">$rep_stat" ) ; 
print  LOGB  "# Repeat\tFound by TRF\tPrimer designed\n" ; 
foreach $period_seq ( sort {$a cmp $b} ( keys ( %{$each_rep_trf_count} ) ) ) { 
	print  LOGB  $period_seq ;
	print  LOGB  "\t".$each_rep_trf_count->{$period_seq} ; 
	if ( $out_primer == 1 ) {
		print  LOGB  "\t".$each_rep_pr3_count->{$period_seq} ;
	} else {
		print  LOGB  "\t---" ;
	}
	print  LOGB  "\n" ; 
}
close ( LOGB ) ; 


open ( LOG, ">>$log" ) ; 
print  LOG  "# ------------------------------------------------------\n" ;
print  LOG  "# Summary of all files :\n" ; 
for ( $u = $min_unit ; $u < $max_unit + 1 ; $u++ ) { 
	print  LOG  "# $u\-bp repeats found : $trf_found->{$u}\n" ;
}
print  LOG  "# Total repeats found by trf : $trf_total\n" ; 
print  LOG  "\n" ;

if ( $out_primer == 1 ) {
	for ( $u = $min_unit ; $u < $max_unit + 1 ; $u++ ) { 
		print  LOG  "# $u\-bp repeat primers designed : $pr3_found->{$u}\n" ;
	}
	print  LOG  "# Total primer pair candidates : $pr3_total\n" ; 
	print  LOG  "\n" ; 
	print  STDERR  "\n" ; 
	print  STDERR  "# Total primer pair candidates : $pr3_total\n" ; 
}
$date = ` date `; 
print  LOG  "# End Date : $date" ; 
close ( LOG ) ;


sub redundancy_check {
	my $seq = uc(shift) ;
	my $len = length($seq) ;
	if ( $len < $red_min ) { 
		if ( defined $red_primer->{$seq} ) { return ( 1 ) ; }
		$red_primer->{$seq} = 1 ; 
		$seq = &s_comp($seq) ;
		if ( defined $red_primer->{$seq} ) { return ( 1 ) ; }
		$red_primer->{$seq} = 1 ;
		return ( 0 ) ; 
	} 
	my $nseq ; 
	my $cseq ;
	for ( my $i = 0 ; $i + $red_min <= $len ; $i++ ) { 
		$nseq = substr ( $seq, $i, $red_min ) ;
		$cseq = &s_comp($nseq) ;
		if ( defined $red_primer->{$nseq} ) { return ( 1 ) ; }
		$red_primer->{$nseq} = 1 ; 
		if ( defined $red_primer->{$cseq} ) { return ( 1 ) ; }
		$red_primer->{$cseq} = 1 ; 
	}
	return ( 0 ) ; 
}

sub s_comp {
	my $seq = shift ;
	my @res = split ( // , $seq ) ;
	my @cres ;
	for ( my $i = @res - 1 ; $i >= 0 ; $i-- ) {
		$res[$i] =~ tr/a-z/A-Z/ ;
		if ( $res[$i] =~ /^[ACGTRYMKSWHBVDN]$/ ) {
		} else {
			$res[$i] = 'N' ;
		}
		if ( defined  $comp_base{ $res[$i] } ) {} else { die  $res[$i]."\n" ; } 
		$cres[ @res - $i - 1 ] = $comp_base{ $res[$i] } ;
	}

	my $cseq = join ( "" , @cres ) ;
	@res = undef ;
	@cres = undef ;
	return ( $cseq ) ;
}





