#!/usr/bin/perl -w

###########################################
# Program : extract_flanking.pl
# Auto-Primer ver 1.00 - Dec. 6, 2012
# Copyright (c) 2012, NRIFS-FRA
# FRA-bioinfo@ml.affrc.go.jp
# http://nrifs.fra.affrc.go.jp/index.html
###########################################

my $min_flanking = 30 ; 	### Minimum length (bp) of flanking sequences
my $opt_pr_len = 22 ; 		### Optimized primer length
my $min_pr_len = 20 ; 
my $max_pr_len = 25 ; 

my $min_product = 100 ;
my $max_product = 400 ;

my $round = 0 ;		### Use default repeat size. 1-> Round to integar

my $out_dir = "." ;
my $out_stem = "primer3_input" ; 

my $in ; 
my $mask = {} ;		### Other repeat regions are not used for primer sequences
my @mask_indices ;
while ( $a = shift(@ARGV) ) { 
	if ( -f $a ) { 
		$in = $a ; 
		if ( $in =~ /.gz$/ ) { 
			open ( IN , "zcat  $in |" ) or die  "$in : Cannot open.\n" ;
			@mask_indices = ` zgrep  \'    Indices\:\' $in ` ; 
		} else { 
			open ( IN , $in ) or die  "$in : Cannot open.\n" ; 
			@mask_indices = ` grep  \'    Indices\:\' $in ` ; 
		}
		if ( @mask_indices < 1 ) { 
			warn  "$in : No repeat found.\n" ; 
			exit(0) ;
		}
		chomp ( @mask_indices ) ; 
		foreach ( @mask_indices ) { 
			/(\d+)\-\-(\d+)/ ;
			for ( $k = $1 ; $k <= $2 ; $k++ ) { $mask->{$k} = 1 ; } 
		}
	} elsif ( $a eq "-round" ) {
		$round = 1 ; 
	} elsif ( $a eq "-min_flanking" && @ARGV > 0 ) { 
		$min_flanking = shift(@ARGV) ;
	} elsif ( $a eq "-opt_len" && @ARGV > 0 ) { 
		$opt_pr_len = shift(@ARGV) ;
	} elsif ( $a eq "-min_len" && @ARGV > 0 ) { 
		$min_pr_len = shift(@ARGV) ;
	} elsif ( $a eq "-max_len" && @ARGV > 0 ) { 
		$max_pr_len = shift(@ARGV) ;
	} elsif ( $a eq "-min_product" && @ARGV > 0 ) { 
		$min_product = shift(@ARGV) ; 
	} elsif ( $a eq "-max_product" && @ARGV > 0 ) { 
		$max_product = shift(@ARGV) ; 
	} elsif ( $a =~ /^\-/ ) { 
		die  "$a : Incorrect option.\n" ;
	} else { 
		$out_dir = $a ;
		if ( ! -d $out_dir ) { mkdir ( $out_dir, 0755 ) ; } 
	}
}
if ( -f $in ) { 
	if ( $in =~ /^.*\/(\S+)/ ) { 
		$out_stem = $out_dir."/".$1 ; 
	} else {
		$out_stem = $out_dir."/".$in ; 
	}
} else {
	die  "File is not defined.\n" ; 
}
if ( $min_product >= $max_product ) { die  "$0 : Product size range is incorrect.\n" ; }
my $product_size_range = $min_product."-".$max_product ;



my $name ; 
while ( <IN> ) {
	chomp ; 
	if ( /^Sequence\:\s*(\S.*\S)/ ) { 
		$name = $1 ; 
		$name =~ s/\s+/\_/g ;
		last ; 
	}
}
print  $name."\n" ; 

my $count = 0 ;
while ( <IN> ) {
	chomp ; 
	### Repeat match
	if ( /^\<A NAME\=\"/ ) { 
	} else {
		next ;
	}
	$count++ ; 

	$line = <IN> ; 
	#print  $line ; 		## Indices
	$line =~ /(\d+)\-\-(\d+)/ ;
	$rep_start = $1 ; 
	$rep_len = $2 - $1 + 1 ;
	
	$line = <IN> ; 
	#print  $line ; 		## Period Size
	$line =~ /Copynumber\:\s*(\S+)\s+Consensus size\:\s*(\d+)/ ;
	$copy = $1 ; 
	$period = $2 ; 
	
	$last_res = "" ;		## For rounding

	$line = <IN> ; 
	$line = <IN> ; 	## Left flanking 10 bp
	$rep = "" ; 
	$gap = 0 ; 
	while ( <IN> ) {
		if ( /\d+\s+(\S.*\S)/ ) {
			$line = $_ ;
			while ( $line =~ /\d+\s+(\S.*\S)/ ) { 
				$r = $1 ; 
				chomp ( $r ) ;
				if ( $r =~ /\s+(\S+)$/ ) { 
					$last_res = $1 ; 
				} else { 
					$last_res = $r ; 
				}
				$r =~ s/\s//g ;
				$rep .= $r ; 
				if ( $r =~ /\-/ ) { $gap++ ; }

				$line = <IN> ;		
				if ( $line =~ /\-/ ) { $gap++ ; }
				
				for ( $i = 0 ; $i < 3 ; $i++ ) { $line = <IN> ; }
			}
			last ; 
		}
	}
	
	### Check the repeat unit
	$period_seq = "" ;
	while ( <IN> ) {
		if ( /^Consensus pattern/ ) { 
			while ( <IN> ) {
				if ( ! /\S/ ) { last ; } 
				chomp ;
				$period_seq .= $_ ;
			}
			last ; 
		} 
	}
	if ( length($period_seq) == 0 ) { warn  "$in : Consensus pattern is not shown.\n" ; }


	$lseq = "" ; 
	$rseq = "" ; 
	$l_start = 0 ; 
	while ( <IN> ) {

		if ( /^Left flanking sequence\:\s*None/ ) { last ; } 
		if ( /^Right flanking sequence\:\s*None/ ) { last ; } 

		if ( /^Left flanking sequence\:\D*(\d+) \-\- (\d+)/ ) {
			$l_start = $1 ;
			$l_end = $2 ;
			while ( <IN> ) {
				if ( ! /\S/ ) { last ; } 
				chomp ;
				$lseq .= $_ ;
			}
		} elsif ( /^Right flanking sequence\:\D*(\d+) \-\- (\d+)/ ) {
			$r_start = $1 ;
			$r_end = $2 ;
			while ( <IN> ) {
				if ( ! /\S/ ) { last ; } 
				chomp ;
				$rseq .= $_ ;
			}
			last ; 
		}
	}
	if ( length($lseq) < $min_flanking ) {
		warn  "$in : No.\=$count : Left flanking sequence is too short (\< $min_flanking bp)\n" ; 
		next ; 
	}
	if ( length($rseq) < $min_flanking ) {
		warn  "$in : No.\=$count : Right flanking sequence is too short (\< $min_flanking bp)\n" ; 
		next ; 
	}


	### Correction of flanking regions by masked repeats
	if ( length($lseq) > 0 ) { 
		$cut = 0 ; 
		for ( $i = $l_end ; $i >= $l_start ; $i-- ) { 
			if ( defined $mask->{$i} ) { last ; } 
			$cut++ ;
		}
#warn  $cut."\n" ; 
		$l_start = length($lseq) - $cut + $l_start ; 
		$lseq = substr ( $lseq, length($lseq) - $cut , $cut ) ;
	}

	if ( length($rseq) > 0 ) { 
		$cut = 0 ; 
		for ( $i = $r_start ; $i <= $r_end ; $i++ ) { 
			if ( defined $mask->{$i} ) { last ; } 
			$cut++ ;
		}
		$rseq = substr ( $rseq, 0, $cut ) ;
	}
	if ( length($lseq) < $min_flanking ) {
		warn  "$in : No.\=$count : Left flanking sequence is too short (\< $min_flanking bp)\n" ; 
		next ; 
	}
	if ( length($rseq) < $min_flanking ) {
		warn  "$in : No.\=$count : Right flanking sequence is too short (\< $min_flanking bp)\n" ; 
		next ; 
	}

	
	### Perfect repeat or imperfect repeat
	$tmp = $rep ; 
	$flag = 0 ;
	$edge = "" ; 
	if ( $copy == int ( $copy ) ) { 
	} else {
		$tmp =~ s/$last_res$// ;
		$edge = ", rounded(".$copy.")" ; 
	}
	while ( $tmp =~ /^\S{$period}/ ) { 
		$tmp = $' ; 
		if ( $& ne $period_seq ) { $flag++ ;  last ; }
	}
	
	if ( $flag == 0 ) { 
		#print  STDERR  "$rep is a perfect repeat of $period_seq\n" ; 
		$judge = "perfect" ; 
	} elsif ( $gap > 0 ) {
		$judge = "imperfect with gaps" ; 
	} else {
		$judge = "imperfect with no gaps" ;
	} 
	$judge .= $edge ;

	$ungap_rep = $rep ; 
	if ( $round == 1 ) { 
		if ( $copy == int ( $copy ) ) { 
		} else {
			$rep =~ s/$last_res$// ;
			$ungap_rep = $` ;
			$ungap_rep =~ s/[^atgcn]//ig ;
			$last_res =~ s/[^atgcn]//ig ;
			$rseq = $last_res.$rseq ;
		}
		if ( length($period_seq) > 9 && $copy < 2 ) { 
		} else {
			$copy = int ( $copy ) ;
		}
	}
	$ungap_rep =~ s/[^atgcn]//ig ;
	#$ungap_rep =~ s/\-//g ;
	if ( length($ungap_rep) == 0 ) {
		warn  "$in : No.\=$count : Failed to repeat sequence\n" ; 
		next ; 
	}
	$period_seq = "(".$period_seq.")".$copy ;

	#print  "Left : $lseq\n" ; 
	#print  "Right : $rseq\n" ; 
	$lseq = lc($lseq) ; 
	$rseq = lc($rseq) ; 
	$full = $lseq.$ungap_rep.$rseq ;
	if ( length($full) < $min_product ) {
		warn  "$in : No.\=$count : Full sequence is too short (\< $min_product bp)\n" ;
		next ; 
	}

	if ( length($lseq) < length($rseq) ) { 
		$flanking_len = length($lseq) ;
	} else {
		$flanking_len = length($rseq) ;
	}


	$out = $out_stem.$count.".seq" ;
	print  join ( "\t", ($out, $l_start, $rep_start, $rep_len, $rep, $period_seq, $period, $judge, $flanking_len, $full) ) ."\n" ; 

	$rep_start = $rep_start - $l_start ;
	$target = $rep_start.",".$rep_len ;
	
	if ( $min_flanking < 0 ) { next ; } 		### Not output a primer3 input

	&output( $out, $name, $full, $target ) ;
	#last ; 
}
close ( IN ) ;



sub output { 
	my $out = shift ; 
	my $id = shift ; 
	my $seq = shift ; 
	my $target = shift ; 
	
open ( OUT, ">$out" ) ; 
print  OUT  << "EOF" ; 
SEQUENCE_ID=$id
SEQUENCE_TEMPLATE=$seq
SEQUENCE_TARGET=$target
PRIMER_TASK=pick_detection_primers
PRIMER_PICK_LEFT_PRIMER=1
PRIMER_PICK_INTERNAL_OLIGO=0
PRIMER_PICK_RIGHT_PRIMER=1
PRIMER_OPT_SIZE=$opt_pr_len
PRIMER_MIN_SIZE=$min_pr_len
PRIMER_MAX_SIZE=$max_pr_len
PRIMER_MAX_NS_ACCEPTED=1
PRIMER_PRODUCT_SIZE_RANGE=$product_size_range
P3_FILE_FLAG=0
SEQUENCE_INTERNAL_EXCLUDED_REGION=$target
PRIMER_EXPLAIN_FLAG=1
=
EOF
close ( OUT ) ;
	
	print  STDERR  "$out is created.\n" ; 
	return ( 1 ) ; 
}

