#!/usr/bin/perl -w

###########################################
# Program : extract_primers.pl
# Auto-Primer ver 1.00 - Dec. 6, 2012
# Copyright (c) 2012, NRIFS-FRA
# FRA-bioinfo@ml.affrc.go.jp
# http://nrifs.fra.affrc.go.jp/index.html
###########################################

my $min_product = 1 ; 	### Minimum amplified length
my $max_product = 500 ; 	### Minimum amplified length


my $fh_in = \*STDIN ;
my $out_stem = "primer3_output" ; 

my $in = "INPUT_HTML_STDIN" ; 
while ( $a = shift (@ARGV) ) {
	if ( $a eq "-min_product" && @ARGV > 0 ) { 
		$min_product = shift (@ARGV) ;
	} elsif ( $a eq "-max_product" && @ARGV > 0 ) { 
		$max_product = shift (@ARGV) ; 
	} elsif ( $a =~ /^\-/ ) { 
		die  "$a : Incorrect option.\n" ;
	}
	if ( -f $a ) { 
		$fh_in = "IN" ;
		$in = $a ; 
		if ( $in =~ /.gz$/ ) { 
			open ( $fh_in , "zcat  $in |" ) or die  "$in : Cannot open.\n" ;
		} else { 
			open ( $fh_in , "$in" ) or die  "$in : Cannot open.\n" ; 
		}
	}
}
if ( $min_product >= $max_product ) { die  "$0 : Product size range is incorrect.\n" ; } 


my $count = 0 ;
while ( <$fh_in> ) {
	chomp ; 
	if ( /^OLIGO/ ) { 
	} else {
		next ;
	}
	$count++ ; 

	$line = <$fh_in> ; 	## Left Primer
	chomp ( $line ) ;
	@data = split ( /\s+/, $line ) ; 
	$l_start = $data[2] + 1 ;		## 5'
	$l_tm = $data[4] ;
	$l_seq = $data[8] ;
	$l_start = $l_start + length($l_seq) - 1 ;	## 3'

	$line = <$fh_in> ; 	## Right Primer
	chomp ( $line ) ;
	@data = split ( /\s+/, $line ) ; 
	$r_start = $data[2] + 1 ;		## 5'
	$r_tm = $data[4] ;
	$r_seq = $data[8] ;
	$r_start = $r_start - length($r_seq) + 1 ;	## 3'
	
	$line = <$fh_in> ; 
	$line = <$fh_in> ; 
	$line = <$fh_in> ; 	
	$line = <$fh_in> ; 	
	$line =~ /^PRODUCT SIZE\:\s*(\d+)/ or die  "$0 : Format error : $line" ;
	$size = $1 ;

	if ( $size < $min_product ) { 
		print  STDERR  "Product size is small \(\<$min_product bp\)\n" ; 
		next ; 
	}
	if ( $size > $max_product ) { 
		print  STDERR  "Product size is large \(\>$max_product bp\)\n" ; 
		next ; 
	}

	print  $size."\t".$l_start."\t".$r_start."\t".$l_tm."\t".$r_tm."\t".$l_seq."\t".$r_seq."\n" ; 

}


